<?php
/**
 * Plugin Name: پلاگین فولفیل ایفا
 * Description: ارسال سفارشات به انبار فول فیلمنت ایفا
 * Version: 1.2.0
 * Author: Fardin Moradi
 * Author URI: https://efaservice.com
 * License: مجوز استفاده از افزونه
 */

defined( 'ABSPATH' ) || exit;

add_action('admin_menu', 'efa_setting_page');

function efa_setting_page() {
    add_options_page(
        'ایفا - تنظیمات',
        'ایفا - تنظیمات',
        'manage_options',
        'efa-service-setting',
        'efaServiceSettingFunction'
    );
}

function efaServiceSettingFunction() {
    ?>
    <div class="wrap">
        <h1>تنظیمات افزرونه ایفا</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('efa_service_filed_settings');
            do_settings_sections('efa-service-setting');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

add_action('admin_init', 'efaServicePluginSetting');

function efaServicePluginSetting() {
    register_setting('efa_service_filed_settings', 'efa_service_selected_order_status', 'sanitize_text_field');
    register_setting('efa_service_filed_settings', 'efa_service_api_username', 'sanitize_text_field');
    register_setting('efa_service_filed_settings', 'efa_service_api_password', 'sanitize_text_field');


    $wooEfaShippingZones = WC_Shipping_Zones::get_zones();
    if ($wooEfaShippingZones) {
        foreach ($wooEfaShippingZones as $wooEfaShippingZone) {
            $zone = new WC_Shipping_Zone($wooEfaShippingZone['zone_id']);
            $zoneMethods = $zone->get_shipping_methods();

            if ($zoneMethods) {
                foreach ($zoneMethods as $methodId => $method) {
                    register_setting('efa_service_filed_settings', 'woo_efa_shipping_' . $methodId, 'sanitize_text_field');
                }
            }
        }
    }

    add_settings_section(
        'efa_service_status_section',
        'انتخاب وضعیت',
        'efaServiceStatusSectionFn',
        'efa-service-setting'
    );

    add_settings_field(
        'efa_service_order_status',
        'وضعیت مورد نظر',
        'efaServiceOrderStatusFn',
        'efa-service-setting',
        'efa_service_status_section'
    );

    add_settings_section(
        'efa_service_api_section',
        'مشخصات API',
        'efaServiceApiFn',
        'efa-service-setting'
    );

    add_settings_field(
        'efa_service_api_username',
        'نام کاربری',
        'efaApiUsernameFn',
        'efa-service-setting',
        'efa_service_api_section'
    );

    add_settings_field(
        'efa_service_api_password',
        'رمز عبور',
        'efaServiceApiPasswordFn',
        'efa-service-setting',
        'efa_service_api_section'
    );

    add_settings_section(
        'efa_service_shipping_section',
        'انتخاب کوریرها',
        'efaServiceShippingFn',
        'efa-service-setting'
    );

    add_settings_field(
        'efa_service_shipping_wc',
        'کوریر شما - کوریر ایفا',
        'efaServiceShippingWcFn',
        'efa-service-setting',
        'efa_service_shipping_section'
    );
}

function efaServiceStatusSectionFn() {
    echo 'سفارش در چه وضعیتی به پلتفرم ایفا ارسال شود: ';
}

function efaServiceOrderStatusFn() {
    $selectedStatus = get_option('efa_service_selected_order_status');
    $orderStatuses = wc_get_order_statuses();

    echo '<select name="efa_service_selected_order_status">';
    foreach ($orderStatuses as $key => $orderStatus) {
        $remove = 'wc-';
        $newKey = $key;

        if (substr($key, 0, strlen($remove)) == $remove) {
            $newKey = substr($key, strlen($remove));
        }
        echo '<option value="' . esc_attr($newKey) . '" ' . selected($selectedStatus, $newKey, false) . '>' . esc_html($orderStatus) . '</option>';
    }
    echo '</select>';
}

function efaServiceApiFn() {
    echo 'رمز عبور و نام کاربری API پلتفرم ایفا: ';
}

function efaApiUsernameFn() {
    $efaServiceApiUsername = get_option('efa_service_api_username');
    echo '<input type="text" name="efa_service_api_username" value="' . esc_attr($efaServiceApiUsername) . '" />';
}

function efaServiceApiPasswordFn() {
    $efaServiceApiPassword = get_option('efa_service_api_password');
    echo '<input type="password" name="efa_service_api_password" value="' . esc_attr($efaServiceApiPassword) . '" />';
}

function efaServiceShippingFn() {
    echo 'مرتبط کردن کوریر های فروشگاه با کوریر های ایفا:';
}

function efaServiceShippingWcFn() {
    $wooEfaShippingZones =  WC_Shipping_Zones::get_zones();
    $efaCouriers = [
        [
            'id' => 1,
            'name' => 'تیپاکس'
        ],
        [
            'id' => 3,
            'name' => 'پست'
        ],
        [
            'id' => 6,
            'name' => 'تحویل درب انبار'
        ],
        [
            'id' => 8,
            'name' => 'الوپیک'
        ],
        [
            'id' => 9,
            'name' => 'اسنپ باکس'
        ],
        [
            'id' => 10,
            'name' => 'اسنپ'
        ],
        [
            'id' => 11,
            'name' => 'لینک اکسپرس'
        ],
        [
            'id' => 12,
            'name' => 'تپسی موتوپیک'
        ],
        [
            'id' => 13,
            'name' => 'تپسی اتوپیک'
        ]
    ];

    if ($wooEfaShippingZones) {
        foreach ($wooEfaShippingZones as $wooEfaShippingZone) {
            $zone = new WC_Shipping_Zone($wooEfaShippingZone['zone_id']);
            $zoneMethods = $zone->get_shipping_methods();

            if ($zoneMethods) {
                foreach ($zoneMethods as $methodId => $method) {
                    echo '<p>';
                    $title = strtok($method->get_title(), '(');
                    $title = preg_replace('/\s+/', ' ', $title);
                    echo '<span style="display: inline-block; width: 350px">' . esc_html($wooEfaShippingZone['zone_name']) . ' - ' . esc_html((strlen($title) > 60) ? substr($title,0,75).'...' : $title) . '</span>';

                    $wooEfaShippingFiled = get_option('woo_efa_shipping_' . $methodId);
                    echo '<select name="woo_efa_shipping_' . $methodId . '">';
                    foreach ($efaCouriers as $efaCourier) {
                        echo '<option value="' . $efaCourier['id'] . '" ' . selected($wooEfaShippingFiled, $efaCourier['id'], false) . '>' . $efaCourier['name'] . '</option>';
                    }
                    echo '</select>';
                    echo '</p>';
                }
            }
        }
    }
}

add_filter('plugin_action_links', 'efaServiceSettingsLink', 10, 2);

function efaServiceSettingsLink($links, $file) {
    if ($file == 'efa-service-plugin/efa-service-plugin.php') {
        $settings_link = '<a href="' . admin_url('options-general.php?page=efa-service-setting') . '">تنظیمات</a>';
        array_push($links, $settings_link);
    }

    return $links;
}

add_action( 'woocommerce_order_status_changed', 'send_to_efa', 10, 30 );

function send_to_efa( $order_id, $old_status, $new_status ) {
    $selectedStatus = get_option('efa_service_selected_order_status');

    if (empty($selectedStatus)) {
        $selectedStatus = 'pws-in-stock';
    }

    if (isset($new_status) && $new_status == $selectedStatus) {
        $efaServiceApiUsername = get_option('efa_service_api_username');
        $efaServiceApiPassword = get_option('efa_service_api_password');

        $order = wc_get_order( $order_id );
        $billing = json_decode($order, true)['billing'];
        $shipping = json_decode($order, true)['shipping'];

        $headers = '{
            "username": "' . $efaServiceApiUsername . '",
            "password": "' . $efaServiceApiPassword . '"
        }';

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://web.efaservice.com/api/v1/login',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $headers,
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/json',
                'Accept: application/json'
            ),
        ));

        $token = curl_exec($curl);

        $wares = null;

        $items = $order->get_items();
        $total = $order->get_total();
        foreach ( $items as $item ) {
            $data = [];
            $product = $item->get_product();
            $data['shop_ware_code'] = $product->get_sku();
            $data['count'] = $item->get_quantity();
            $data['per_price'] = ($item->get_total() / $item->get_quantity()) * 10;

            $wares[] = $data;
        }

        $shippingId = null;
        $shippingItems = $order->get_items('shipping');
        if ($shippingItems) {
            foreach ($shippingItems as $shippingItem) {
                $shippingId = $shippingItem->get_instance_id();
                break;
            }
        }

        $courierId = $shippingId
            ? get_option('woo_efa_shipping_' . $shippingId)
            : 3;

        $file_path = plugin_dir_path( __FILE__ ) . 'data/cities.json';

        $file = fopen($file_path, 'r');
        $provinces = fread($file, filesize($file_path));
        fclose($file);

        $provinces = json_decode($provinces, true);

        $provinceCode = $provinces[$order->get_shipping_state()]['code'];
        $cityCode = null;
        foreach ($provinces[$order->get_shipping_state()]['cities'] as $cityKey => $city) {
            if ($order->get_shipping_city() === $city) {
                $cityCode = $cityKey;
                break;
            }
        }
        $url = 'https://web.efaservice.com/api/v1/orders-code';
        if ($cityCode === null) {
            $url = 'https://web.efaservice.com/api/v1/orders-city-name-code';
            $provinceCode = $provinces[$order->get_shipping_state()]['code'];
            $cityCode = $order->get_shipping_city();
        }

        $data = [
            "shop_order_code"=> $order->ID,
            "no_delivery" => false,
            "first_name" => $shipping['first_name'],
            "last_name" => $shipping['last_name'],
            "province_id" => $provinceCode,
            "city_id" => $cityCode,
            "info" => [
                "mobile" => $billing['phone'],
                "address" => $shipping['address_1'] . ' - ' . $shipping['address_2'],
                "unit" => "0",
                "no" => "0",
                "zip_code" => $shipping['postcode']
            ],
            "has_packing" => true,
            "has_cod" => false,
            "has_pod" => false,
            "cod_price" => $total,
            "courier_id" => $courierId,
            "delivery_at" => date("Y-m-d H:i:s", time() + 86400),
            "wares" => $wares
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 0);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data, true));
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Accept: application/json',
            'Content-Type: application/json',
            'Authorization: ' . $token
        ));

        $httpCode = curl_getinfo($ch , CURLINFO_HTTP_CODE); // this results 0 every time
        $response = curl_exec($ch);

        if (!isset(json_decode($response)->id)) {
            $order->update_status( 'pws-packaged', 'وضعیت تغییر نکرد سفارش شما به انبار ایفا ارسال نشد.' );
            $errors = json_decode($response, true)['errors'];
            foreach ($errors as $error) {
                $message = $shipping['first_name'] . ' ' . $shipping['last_name'] . ': ' . $error[0];
                wp_die($message);
            }
        }

        curl_close($ch);
    }
}
?>